==> chainwallet-1.0.2.tar.gz: lib/src/core/coretypes/hash/Hash.dart <==
import "dart:typed_data";

import 'package:chainwallet/src/core/serialized/SerializedType.dart';
import 'package:chainwallet/src/core/serialized/TypeTranslator.dart';

import 'package:chainwallet/src/utils/Util.dart';
import 'package:collection/collection.dart';

import 'package:chainwallet/src/core/serialized/BinaryParser.dart';

import 'package:chainwallet/src/core/serialized/BytesSink.dart';




//  foo
// abstract public class UInt<Subclass extends UInt> extends Number implements SerializedType, Comparable<UInt> {

abstract class Hash<T> implements SerializedType{

  Uint8List hash;
  int hashCode = -1;

  Hash(Uint8List bytes, int size) {
        hash = normalizeAndCheckHash(bytes, size);
  }

  Uint8List normalizeAndCheckHash(Uint8List bytes, int size) {
        int length = bytes.length;
        if (length > size) {
            String simpleName = "";
            throw new Exception("Hash length of " + length.toString() + "  is too wide for " + simpleName);
        }
        if (length == size) {

            return bytes;
        } else {
            Uint8List hash = new Uint8List(size);
            Util.arraycopy(bytes, 0, hash, size - length, length);
            return hash;
        }
  }

  String toString() {

    String s = new String.fromCharCodes(hash);

    return s;
  }

  // TODO
  int getHashCode() {
    if (hashCode == -1) {
      String s = new String.fromCharCodes(hash);
      hashCode = s.hashCode;
    }
    return hashCode;
  }


  BigInt bigInteger() {
    return Util.createBigIntFromUint8List(hash);
  }

  Uint8List bytes() {
    return hash;
  }

  bool equals(Object obj) {
    if (obj.runtimeType == Hash) {

      Function eq = const ListEquality().equals;
      //  print(eq([1,'two',3], [1,'two',3])); // => true
      var objHash = obj as Hash;
      return  eq(objHash.hash,this.hash);

    }
    // TODO
    return this == obj;
  }

  int compareTo(T another) {
    Uint8List thisBytes = this.bytes();
    var anotherHash = another as Hash;
    Uint8List bytes = anotherHash.bytes();

    return compareBytes(thisBytes, bytes, 0, thisBytes.length);
  }

  int compareStartingAt(T another, int start) {
    Uint8List thisBytes = this.bytes();
    var anotherHash = another as Hash;
    Uint8List bytes = anotherHash.bytes();
    return compareBytes(thisBytes, bytes, start, thisBytes.length);
  }

  int compareBytes(Uint8List thisBytes, Uint8List bytes, int start, int numBytes) {
        int thisLength = thisBytes.length;
        if (!(bytes.length == thisLength)) {
            throw new Exception();
        }

        for (int i = start; i < numBytes; i++) {
        int cmp = (thisBytes[i] & 0xFF) - (bytes[i] & 0xFF);
        if (cmp != 0) {
        return cmp < 0 ? -1 : 1;
        }
        }
        return 0;
  }


  Uint8List slice(int start) {
    return sliceRange(start, 0);
  }

  Uint8List sliceRange(int start, int end) {
    if (start < 0)  start += hash.length;
    if (end  <= 0)  end   += hash.length;

    int length = end - start;
    Uint8List slice = new Uint8List(length);

    Util.arraycopy(hash, start, slice, 0, length);
    return slice;
  }


  int get(int i) {
    if (i < 0) i += hash.length;
    return hash[i];
  }


}

abstract class HashTranslator<T extends Hash> extends TypeTranslator<T> {

        T newInstance(Uint8List b);
        int byteWidth();
        //
        //  @Override
          T fromParser(BinaryParser parser, int hint) {
            return newInstance(parser.read(byteWidth()));
          }
        //
        //  TODO  hash 和Object中的重复了
          Map toJSON(T obj) {
            return  null;
          }
        //
        //  @Override
          T fromString(String value) {
            return newInstance(Util.createUint8ListFromString(value));
          }
        //
        //  @Override
          void toBytesSink(T obj, BytesSink to) {
            to.addBytes(obj.hash);
          }
}

==> chainwallet-1.0.4.tar.gz: lib/src/core/coretypes/hash/Hash.dart <==
import "dart:typed_data";

import 'package:chainwallet/src/core/serialized/SerializedType.dart';
import 'package:chainwallet/src/core/serialized/TypeTranslator.dart';

import 'package:chainwallet/src/utils/Util.dart';
import 'package:collection/collection.dart';

import 'package:chainwallet/src/core/serialized/BinaryParser.dart';

import 'package:chainwallet/src/core/serialized/BytesSink.dart';

//  foo
// abstract public class UInt<Subclass extends UInt> extends Number implements SerializedType, Comparable<UInt> {

abstract class Hash<T> implements SerializedType {
  Uint8List hash;
  int hashCode = -1;

  Hash(Uint8List bytes, int size) {
    hash = normalizeAndCheckHash(bytes, size);
  }

  Uint8List normalizeAndCheckHash(Uint8List bytes, int size) {
    int length = bytes.length;
    if (length > size) {
      String simpleName = "";
      throw new Exception("Hash length of " +
          length.toString() +
          "  is too wide for " +
          simpleName);
    }
    if (length == size) {
      return bytes;
    } else {
      Uint8List hash = new Uint8List(size);
      Util.arraycopy(bytes, 0, hash, size - length, length);
      return hash;
    }
  }

  String toString() {
    String s = new String.fromCharCodes(hash);

    return s;
  }

  // TODO
  int getHashCode() {
    if (hashCode == -1) {
      String s = new String.fromCharCodes(hash);
      hashCode = s.hashCode;
    }
    return hashCode;
  }

  BigInt bigInteger() {
    return Util.createBigIntFromUint8List(hash);
  }

  Uint8List bytes() {
    return hash;
  }

  bool equals(Object obj) {
    if (obj.runtimeType == Hash) {
      Function eq = const ListEquality().equals;
      //  print(eq([1,'two',3], [1,'two',3])); // => true
      var objHash = obj as Hash;
      return eq(objHash.hash, this.hash);
    }
    // TODO
    return this == obj;
  }

  int compareTo(T another) {
    Uint8List thisBytes = this.bytes();
    var anotherHash = another as Hash;
    Uint8List bytes = anotherHash.bytes();

    return compareBytes(thisBytes, bytes, 0, thisBytes.length);
  }

  int compareStartingAt(T another, int start) {
    Uint8List thisBytes = this.bytes();
    var anotherHash = another as Hash;
    Uint8List bytes = anotherHash.bytes();
    return compareBytes(thisBytes, bytes, start, thisBytes.length);
  }

  int compareBytes(
      Uint8List thisBytes, Uint8List bytes, int start, int numBytes) {
    int thisLength = thisBytes.length;
    if (!(bytes.length == thisLength)) {
      throw new Exception();
    }

    for (int i = start; i < numBytes; i++) {
      int cmp = (thisBytes[i] & 0xFF) - (bytes[i] & 0xFF);
      if (cmp != 0) {
        return cmp < 0 ? -1 : 1;
      }
    }
    return 0;
  }

  Uint8List slice(int start) {
    return sliceRange(start, 0);
  }

  Uint8List sliceRange(int start, int end) {
    if (start < 0) start += hash.length;
    if (end <= 0) end += hash.length;

    int length = end - start;
    Uint8List slice = new Uint8List(length);

    Util.arraycopy(hash, start, slice, 0, length);
    return slice;
  }

  int get(int i) {
    if (i < 0) i += hash.length;
    return hash[i];
  }
}

abstract class HashTranslator<T extends Hash> extends TypeTranslator<T> {
  T newInstance(Uint8List b);
  int byteWidth();
  //
  //  @Override
  T fromParser(BinaryParser parser, int hint) {
    return newInstance(parser.read(byteWidth()));
  }

  //
  //  TODO  hash 和Object中的重复了
  Map toJSON(T obj) {
    return null;
  }

  //
  //  @Override
  T fromString(String value) {
    return newInstance(Util.createUint8ListFromString(value));
  }

  //
  //  @Override
  void toBytesSink(T obj, BytesSink to) {
    to.addBytes(obj.hash);
  }
}
